/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network:Interface
 * Copyright (c) 2004-2021 Arm Limited (or its affiliates). All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    Net_Config_WiFi_%Instance%.h
 * Purpose: Network Configuration for WiFi Interface
 * Rev.:    V7.2.0
 *----------------------------------------------------------------------------*/

//-------- <<< Use Configuration Wizard in Context Menu >>> --------------------

// <h>WiFi Network Interface
#define WIFI%Instance%_ENABLE             1

//   <o>Connect to hardware via Driver_WiFi# <0-255>
//   <i>Select driver control block for WiFi interface
#define WIFI%Instance%_DRIVER             0

//   <o>WiFi Mode <0=>Station
//                <1=>Access Point
//   <i>Mode control setting for the WiFi driver
#define WIFI%Instance%_MODE               0

//   <e>WiFi MAC
//   <i>If this option is enabled, a configured MAC address
//   <i>will be used. If disabled, MAC address provided in
//   <i>the WiFi module will be used.
//   <i>Default: disabled
#define WIFI%Instance%_MAC_CONFIG         0

//     <s.17>MAC Address
//     <i>MAC Address in text representation
//     <i>Value FF-FF-FF-FF-FF-FF is not allowed,
//     <i>LSB of first byte must be 0 (Multicast bit).
//     <i>Default: "1E-30-6C-32-78-F8"
#define WIFI%Instance%_MAC_ADDR           "1E-30-6C-32-78-F8"
//   </e>

//   <e>IPv4
//   <i>Enable IPv4 Protocol for Network Interface
#define WIFI%Instance%_IP4_ENABLE         1

//     <s.15>IP Address
//     <i>Static IPv4 Address in text representation
//     <i>Default: "192.168.1.100"
#define WIFI%Instance%_IP4_ADDR           "192.168.1.100"

//     <s.15>Subnet mask
//     <i>Local Subnet mask in text representation
//     <i>Default: "255.255.255.0"
#define WIFI%Instance%_IP4_MASK           "255.255.255.0"

//     <s.15>Default Gateway
//     <i>IP Address of Default Gateway in text representation
//     <i>Default: "192.168.1.254"
#define WIFI%Instance%_IP4_GATEWAY        "192.168.1.254"

//     <s.15>Primary DNS Server
//     <i>IP Address of Primary DNS Server in text representation
//     <i>Default: "8.8.8.8"
#define WIFI%Instance%_IP4_PRIMARY_DNS    "8.8.8.8"

//     <s.15>Secondary DNS Server
//     <i>IP Address of Secondary DNS Server in text representation
//     <i>Default: "8.8.4.4"
#define WIFI%Instance%_IP4_SECONDARY_DNS  "8.8.4.4"

//     <e>IP Fragmentation
//     <i>This option enables fragmentation of outgoing IP datagrams,
//     <i>and reassembling the fragments of incoming IP datagrams.
//     <i>Default: enabled
#define WIFI%Instance%_IP4_FRAG_ENABLE    1

//       <o>MTU size <576-1500>
//       <i>Maximum Transmission Unit in bytes
//       <i>Default: 1500
#define WIFI%Instance%_IP4_MTU            1500
//     </e>

//     <h>ARP Address Resolution
//     <i>ARP cache and node address resolver settings
//       <o>Cache Table size <5-100>
//       <i>Number of cached MAC/IP addresses
//       <i>Default: 10
#define WIFI%Instance%_ARP_TAB_SIZE       10

//       <o>Cache Timeout in seconds <5-255>
//       <i>A timeout for cached hardware/IP addresses
//       <i>Default: 150
#define WIFI%Instance%_ARP_CACHE_TOUT     150

//       <o>Number of Retries <0-20>
//       <i>Number of Retries to resolve an IP address
//       <i>before ARP module gives up
//       <i>Default: 4
#define WIFI%Instance%_ARP_MAX_RETRY      4

//       <o>Resend Timeout in seconds <1-10>
//       <i>A timeout to resend the ARP Request
//       <i>Default: 2
#define WIFI%Instance%_ARP_RESEND_TOUT    2

//       <q>Send Notification on Address changes
//       <i>When this option is enabled, the embedded host
//       <i>will send a Gratuitous ARP notification at startup,
//       <i>or when the device IP address has changed.
//       <i>Default: Disabled
#define WIFI%Instance%_ARP_NOTIFY         0
//     </h>

//     <e>IGMP Group Management
//     <i>Enable or disable Internet Group Management Protocol
#define WIFI%Instance%_IGMP_ENABLE        0

//       <o>Membership Table size <2-50>
//       <i>Number of Groups this host can join
//       <i>Default: 5
#define WIFI%Instance%_IGMP_TAB_SIZE      5
//     </e>

//     <q>NetBIOS Name Service
//     <i>When this option is enabled, the embedded host can be
//     <i>accessed by its name on local LAN using NBNS protocol.
#define WIFI%Instance%_NBNS_ENABLE        1

//     <e>Dynamic Host Configuration
//     <i>When this option is enabled, local IP address, Net Mask
//     <i>and Default Gateway are obtained automatically from
//     <i>the DHCP Server on local LAN.
#define WIFI%Instance%_DHCP_ENABLE        1

//       <s.40>Vendor Class Identifier
//       <i>This value is optional. If specified, it is added
//       <i>to DHCP request message, identifying vendor type.
//       <i>Default: ""
#define WIFI%Instance%_DHCP_VCID          ""

//       <q>Bootfile Name
//       <i>This value is optional. If enabled, the Bootfile Name
//       <i>(option 67) is also requested from DHCP server.
//       <i>Default: disabled
#define WIFI%Instance%_DHCP_BOOTFILE      0

//       <q>NTP Servers
//       <i>This value is optional. If enabled, a list of NTP Servers
//       <i>(option 42) is also requested from DHCP server.
//       <i>Default: disabled
#define WIFI%Instance%_DHCP_NTP_SERVERS   0
//     </e>

//     Disable ICMP Echo response
#define WIFI%Instance%_ICMP_NO_ECHO       0
//   </e>

//   <e>IPv6
//   <i>Enable IPv6 Protocol for Network Interface
#define WIFI%Instance%_IP6_ENABLE         1

//     <s.40>IPv6 Address
//     <i>Static IPv6 Address in text representation
//     <i>Use unspecified address "::" when static
//     <i>IPv6 address is not used.
//     <i>Default: "fec0::2"
#define WIFI%Instance%_IP6_ADDR           "fec0::2"

//     <o>Subnet prefix-length <1-128>
//     <i>Number of bits that define network address
//     <i>Default: 64
#define WIFI%Instance%_IP6_PREFIX_LEN     64

//     <s.40>Default Gateway
//     <i>Default Gateway IPv6 Address in text representation
//     <i>Default: "fec0::1"
#define WIFI%Instance%_IP6_GATEWAY        "fec0::1"

//     <s.40>Primary DNS Server
//     <i>Primary DNS Server IPv6 Address in text representation
//     <i>Default: "2001:4860:4860::8888"
#define WIFI%Instance%_IP6_PRIMARY_DNS    "2001:4860:4860::8888"

//     <s.40>Secondary DNS Server
//     <i>Secondary DNS Server IPv6 Address in text representation
//     <i>Default: "2001:4860:4860::8844"
#define WIFI%Instance%_IP6_SECONDARY_DNS  "2001:4860:4860::8844"

//     <e>IPv6 Fragmentation
//     <i>This option enables fragmentation of outgoing IPv6 datagrams,
//     <i>and reassembling the fragments of incoming IPv6 datagrams.
//     <i>Default: enabled
#define WIFI%Instance%_IP6_FRAG_ENABLE    1

//       <o>MTU size <1280-1500>
//       <i>Maximum Transmission Unit in bytes
//       <i>Default: 1500
#define WIFI%Instance%_IP6_MTU            1500
//     </e>

//     <h>Neighbor Discovery
//     <i>Neighbor cache and node address resolver settings
//       <o>Cache Table size <5-100>
//       <i>Number of cached node addresses
//       <i>Default: 5
#define WIFI%Instance%_NDP_TAB_SIZE       5

//       <o>Cache Timeout in seconds <5-255>
//       <i>Timeout for cached node addresses
//       <i>Default: 150
#define WIFI%Instance%_NDP_CACHE_TOUT     150

//       <o>Number of Retries <0-20>
//       <i>Number of retries to resolve an IP address
//       <i>before NDP module gives up
//       <i>Default: 4
#define WIFI%Instance%_NDP_MAX_RETRY      4

//       <o>Resend Timeout in seconds <1-10>
//       <i>A timeout to resend Neighbor Solicitation
//       <i>Default: 2
#define WIFI%Instance%_NDP_RESEND_TOUT    2
//     </h>

//     <e>Dynamic Host Configuration
//     <i>When this option is enabled, local IPv6 address is
//     <i>automatically configured.
#define WIFI%Instance%_DHCP6_ENABLE       1

//       <o>DHCPv6 Client Mode  <0=>Stateless Mode <1=>Statefull Mode
//       <i>Stateless DHCPv6 Client uses router advertisements
//       <i>for IPv6 address autoconfiguration (SLAAC).
//       <i>Statefull DHCPv6 Client connects to DHCPv6 server for a
//       <i>leased IPv6 address and DNS server IPv6 addresses.
#define WIFI%Instance%_DHCP6_MODE         1

//       <e>Vendor Class Option
//       <i>If enabled, Vendor Class option is added to DHCPv6
//       <i>request message, identifying vendor type.
//       <i>Default: disabled
#define WIFI%Instance%_DHCP6_VCLASS_ENABLE 0

//         <o>Enterprise ID
//         <i>Enterprise-number as registered with IANA.
//         <i>Default: 0 (Reserved)
#define WIFI%Instance%_DHCP6_VCLASS_EID   0

//         <s.40>Vendor Class Data
//         <i>This string identifies vendor type.
//         <i>Default: ""
#define WIFI%Instance%_DHCP6_VCLASS_DATA  ""
//       </e>
//     </e>

//     Disable ICMP6 Echo response
#define WIFI%Instance%_ICMP6_NO_ECHO      0
//   </e>

//   <h>OS Resource Settings
//   <i>These settings are used to optimize usage of OS resources.
//     <o>Interface Thread Stack Size <512-65535:4>
//     <i>Default: 512 bytes
#define WIFI%Instance%_THREAD_STACK_SIZE  512

//        Interface Thread Priority
#define WIFI%Instance%_THREAD_PRIORITY    osPriorityAboveNormal

//   </h>
// </h>

//------------- <<< end of configuration section >>> ---------------------------
